/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for Petit FatFs (C)ChaN, 2009      */
/*-----------------------------------------------------------------------*/

#include "diskio.h"
#include "sd.h"



/*-----------------------------------------------------------------------*/
/* Initialize Disk Drive                                                 */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize (void)
{
	DSTATUS stat;

	MMC_SD_Init();
	
	stat = MMC_SD_Reset();	
	return stat;
}



/*-----------------------------------------------------------------------*/
/* Read Partial Sector                                                   */
/*-----------------------------------------------------------------------*/

DRESULT disk_readp (
	BYTE* buffer,			/* Pointer to the destination object */
	DWORD sector,		/* Sector number (LBA) */
	WORD sofs,			/* Offset in the sector */
	WORD count			/* Byte count (bit15:destination) */
)
{
	DRESULT res;
	uint8 r1;
	uint16 bc;
	
	res = RES_ERROR;

	r1 = MMC_SD_SendCommand(17, sector<<9);//
	if(r1 != 0x00)
		return res;
		
	SPI_CS_Assert();	
	
	bc = 0;
	do 
	{							/* Wait for data packet */
		if(bc++ > 1000)	{SPI_CS_Deassert();return res;}
		
	}while(SPI_WriteByte(0xff) != 0xfe);	//ݵĿʼ

	/* Skip leading bytes */
	if (sofs) 
	{
		bc = sofs;
		do
		{
			SPI_WriteByte(0xff);
		} 
		while(--bc);
	}
	if(buffer) /* Store data to the memory */
	{	
		bc = count;
		do 
		{
			*(buffer++)=SPI_WriteByte(0xff);
		} while (--bc);
	} 
	else /* Forward data to the outgoing stream (depends on the project) */
	{
		bc = count;
		do 
		{
			SPI_WriteByte(0xff);
		} while (--bc);
	}
	
	/* Skip trailing bytes and CRC */
	bc = 514 - sofs - count;
	do 
	{
		SPI_WriteByte(0xff); 
	}while (--bc);

	SPI_CS_Deassert();
	res = RES_OK;	
	return res;
}



/*-----------------------------------------------------------------------*/
/* Write Partial Sector                                                  */
/*-----------------------------------------------------------------------*/

DRESULT disk_writep (
	const BYTE* buffer,		/* Pointer to the data to be written, NULL:Initiate/Finalize write operation */
	DWORD sc		/* Sector number (LBA) or Number of bytes to send */
)
{
	DRESULT res;
	uint8 r1;
	uint16 i;
	
	res = RES_ERROR;
	if (!buffer) 
	{
		if (sc) 
		{
			r1 = MMC_SD_SendCommand(24, sc<<9);//дInitiate write process
			if(r1 != 0x00)
				return res;

			SPI_CS_Assert();

			SPI_WriteByte(0xff);
			SPI_WriteByte(0xff);
			SPI_WriteByte(0xff);

			SPI_WriteByte(0xfe);//ʼ// 
			res = RES_OK;

		} 
		else // Finalize write process
		{
			SPI_WriteByte(0xff);
			SPI_WriteByte(0xff);
			r1 = SPI_WriteByte(0xff);

			if( (r1&0x1f) != 0x05)//ȴǷɹ
			{
				SPI_CS_Deassert();
				return res;
			}
			//ȴ
			while(!SPI_WriteByte(0xff));

			SPI_CS_Deassert();	
			res = RES_OK;		
		}
	} 
	else 
	{

		for(i=0; i<64; i++)//512ֽ
		{
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
			SPI_WriteByte(*buffer++);
		}// Send data to the disk
		res = RES_OK;
	}

	return res;
}

